package ed20081.g1.tp1.aleatorios;

import static ed20081.g1.tp1.aleatorios.RandomUtils.escolherElemento;
import static ed20081.g1.tp1.aleatorios.RandomUtils.gerarBoolean;
import static ed20081.g1.tp1.aleatorios.RandomUtils.gerarIntNoIntervaloFechado;

import java.util.Date;

import ed20081.g1.tp1.codigo.Sinistro;
import ed20081.g1.tp1.codigo.TipoSinistro;
import ed20081.g1.tp1.enums.TiposPesquisa;
import ed20081.g1.tp1.enums.aleatorios.Descricoes;
import ed20081.g1.tp1.enums.aleatorios.Enderecos;
import ed20081.g1.tp1.enums.aleatorios.Estados;
import ed20081.g1.tp1.enums.aleatorios.Municipios;
import ed20081.g1.tp1.excecoes.ExcecaoPosicaoInvalida;
import ed20081.g1.tp1.persistencia.SerializadorSinistro;
import ed20081.g1.tp1.persistencia.SerializadorVeiculo;
import ed20081.g1.tp1.pesquisa.GerenciadorDeIndices;
import ed20081.g1.tp1.principal.Opcoes;

public class GeradorAleatorioDeSinistros extends GeradorAleatorio {

	private final SerializadorSinistro serializadorSinistro;
	private final SerializadorVeiculo serializadorDeVeiculo;
	private final long numeroDeVeiculos;

	public GeradorAleatorioDeSinistros(SerializadorSinistro serializador) {
		super();
		serializadorSinistro = serializador;
		serializadorDeVeiculo = SerializadorVeiculo.obterInstancia();
		numeroDeVeiculos = serializadorDeVeiculo.contarEntradas();
	}

	public void cadastrarAleatoriamenteN(int n) throws ExcecaoPosicaoInvalida {
		TiposPesquisa tipoPesquisa = Opcoes.obterTipoPesquisa();
		int codigo = serializadorSinistro.obterProximoCodigo();

		for (int inseridos = 0; inseridos < n; inseridos++) {
			String placa = "";

			placa = gerarPlaca();
			final Date data = gerarDataAleatoria();
			final String municipio = escolherElemento(Municipios.valores).toString();
			final String estado = escolherElemento(Estados.values()).toString();
			final String endereco = escolherElemento(Enderecos.valores).toString();
			final String descricao = escolherElemento(Descricoes.valores).toString();
			final boolean motoristaEmbriagado = gerarBoolean();
			final boolean perdaTotal = gerarBoolean();
			final int vitimasNaoFatais = gerarIntNoIntervaloFechado(0, 99);
			final int vitimasFatais = gerarIntNoIntervaloFechado(0, 99);

			final TipoSinistro sin = new Sinistro(codigo, placa, data, municipio, estado, endereco,
					motoristaEmbriagado, perdaTotal, vitimasNaoFatais, vitimasFatais, descricao);

			serializadorSinistro.serializar(sin);
			if (tipoPesquisa == TiposPesquisa.INDEXADA)
				GerenciadorDeIndices.adicionarNosIndicesDeSinistro(sin);
			codigo++;
		}
	}

	private String gerarPlaca() throws ExcecaoPosicaoInvalida {
		final int posioDoSorteado = (int) RandomUtils.gerarLongNoIntervaloFechado(1,
				numeroDeVeiculos);
		// TODO Traduzindo desnecessariamente (SinistrosAleatorios mais lentos)
		final String placa = serializadorDeVeiculo.lerVeiculoDaPosicao(posioDoSorteado)
				.obterPlaca();
		return placa;
	}

}
