package ed20081.g1.tp1.persistencia;

import static ed20081.g1.tp1.util.StringUtils.imprimirStringConvertida;

import java.io.EOFException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

import ed20081.g1.tp1.excecoes.ExcecaoPosicaoInvalida;

public abstract class Serializador {

	protected File f;
	private int tamanhoDoSerializado;
	private int posicao;

	public boolean acabouArquivo() {
		return getPosicao() * tamanhoDoSerializado == f.length();
	}

	public long contarEntradas() {
		return f.length() / tamanhoDoSerializado;
	}

	public void fixarArquivo(File f) {
		this.f = f;
	}

	public void fixarArquivo(String arquivo) {
		f = new File(arquivo);
	}

	public void fixarTamanhoDoSerializado(int tamanho) {
		tamanhoDoSerializado = tamanho;
	}

	public int getPosicao() {
		return posicao;
	}

	public String lerDaPosicao(int posicao) throws ExcecaoPosicaoInvalida {

		if (posicao < 1 || posicao * tamanhoDoSerializado > f.length())
			throw new ExcecaoPosicaoInvalida();

		String achado = "";
		try {
			final FileChannel channel = new RandomAccessFile(f, "rw").getChannel();
			final ByteBuffer bg = channel.map(FileChannel.MapMode.READ_WRITE,
					((posicao - 1) * tamanhoDoSerializado), tamanhoDoSerializado);

			final byte[] lido = new byte[tamanhoDoSerializado];
			bg.get(lido);
			achado = new String(lido, "ISO8859-1");

			channel.close();
		}

		catch (final EOFException e) {
			imprimirStringConvertida("Tentou ler alm do fim do arquivo!");
		} catch (final FileNotFoundException e) {
			imprimirStringConvertida("Arquivo nao encontrado!");
		} catch (final IOException e) {
			imprimirStringConvertida("Voc no tem permisso para acessar este arquivo!");
		}

		return achado;

	}

	public String[] lerDoArquivo(int numeroDeObjetos) {
		String[] retornados = null;

		if (getPosicao() * tamanhoDoSerializado + tamanhoDoSerializado * numeroDeObjetos > f
				.length())
			numeroDeObjetos = (int) (contarEntradas() - getPosicao());

		// TODO Verificar se o clculo est correto

		retornados = new String[numeroDeObjetos];

		try {
			final FileChannel channel = new RandomAccessFile(f, "rw").getChannel();
			final ByteBuffer bg = channel
					.map(FileChannel.MapMode.READ_WRITE, (getPosicao() * tamanhoDoSerializado),
							(tamanhoDoSerializado * numeroDeObjetos));

			for (int i = 0; i < retornados.length; i++) {
				final byte[] lido = new byte[tamanhoDoSerializado];
				bg.get(lido);
				retornados[i] = new String(lido, "ISO8859-1");
			}

			channel.close();
			setPosicao(getPosicao() + numeroDeObjetos);
		}

		catch (final EOFException e) {
			imprimirStringConvertida("Fim do arquivo!");
		} catch (final FileNotFoundException e) {
			imprimirStringConvertida("Arquivo nao encontrado!");
		} catch (final IOException e) {
			imprimirStringConvertida("Voc no tem permisso para acessar este arquivo!");
		}

		return retornados;
	}

	public void limparArquivo() {
		String temp = f.getAbsolutePath();
		temp = f.getName();
		f.delete();
		f = new File(temp);
	}

	public void removerLinha(int posicaoDaLinha) {
		try {
			final FileChannel channel = new RandomAccessFile(f, "rw").getChannel();
			final ByteBuffer ultimo = channel.map(FileChannel.MapMode.READ_WRITE,
					(f.length() - tamanhoDoSerializado), tamanhoDoSerializado);

			final ByteBuffer linhaSobrescrita = channel.map(FileChannel.MapMode.READ_WRITE,
					(posicaoDaLinha - 1) * tamanhoDoSerializado, tamanhoDoSerializado);

			linhaSobrescrita.put(ultimo);

			channel.truncate(f.length() - tamanhoDoSerializado);
			channel.close();
		} catch (final FileNotFoundException e) {
			e.printStackTrace();
			imprimirStringConvertida("Arquivo no encontrado");
		} catch (final EOFException e) {
			e.printStackTrace();
			imprimirStringConvertida("Fim do arquivo");
		} catch (final IOException e) {
			e.printStackTrace();
			imprimirStringConvertida("Voc no tem permisso para acessar este arquivo");
		}
	}

	public void salvarNaPosicao(int posicaoDoEncontrado, String serializado) {
		try {
			final FileChannel channel = new RandomAccessFile(f, "rw").getChannel();
			final ByteBuffer bb = channel.map(FileChannel.MapMode.READ_WRITE,
					((posicaoDoEncontrado - 1) * serializado.length()), serializado.length());
			bb.put(serializado.getBytes("ISO8859-1"));
			channel.close();

		} catch (final FileNotFoundException e) {
			imprimirStringConvertida("Arquivo no encontrado");
		} catch (final EOFException e) {
			imprimirStringConvertida("Fim do arquivo");
		} catch (final IOException e) {
			imprimirStringConvertida("Voc no tem permisso para acessar este arquivo");
		}

	}

	public void serializar(String serializado) {

		try {
			final FileChannel channel = new RandomAccessFile(f, "rw").getChannel();
			final ByteBuffer bb = channel.map(FileChannel.MapMode.READ_WRITE, f.length(),
					serializado.length());
			bb.put(serializado.getBytes("ISO8859-1"));
			channel.close();

		} catch (final FileNotFoundException e) {
			imprimirStringConvertida("Arquivo no encontrado");
		} catch (final EOFException e) {
			imprimirStringConvertida("Fim do arquivo");
		} catch (final IOException e) {
			imprimirStringConvertida("Voc no tem permisso para acessar este arquivo");

		}

	}

	public void setPosicao(int posicao) {
		this.posicao = posicao;
	}

}
