package ed20081.g1.tp1.persistencia;

import static ed20081.g1.tp1.util.StringUtils.imprimirStringConvertida;
import ine5408.lineares.Lista;
import ine5408.pp.Iterador;
import ine5408.th.TabelaDeHash;

import java.io.EOFException;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

import ed20081.g1.tp1.codigo.Sinistro;
import ed20081.g1.tp1.codigo.TipoSinistro;
import ed20081.g1.tp1.enums.TiposPesquisa;
import ed20081.g1.tp1.excecoes.ExcecaoPosicaoInvalida;
import ed20081.g1.tp1.pesquisa.GerenciadorDeIndices;
import ed20081.g1.tp1.principal.Opcoes;
import ed20081.g1.tp1.util.ListaDuplamenteEncadeada;

public class SerializadorSinistro extends Serializador {

	private static SerializadorSinistro instanciaUnica;

	public static SerializadorSinistro obterInstancia() {
		if (instanciaUnica == null)
			instanciaUnica = new SerializadorSinistro();

		return instanciaUnica;
	}

	private final Tradutor tradutor = Tradutor.obterInstancia();
	private final int tamanhoDeUmSinistro = tradutor.obterTamanhoSinistro();
	private final int tamanhoPlaca = tradutor.obterTamanhoPlaca();
	private final int tamanhoCodigo = tradutor.obterTamanhoCodigo();

	private SerializadorSinistro() {
		fixarTamanhoDoSerializado(tamanhoDeUmSinistro);
	}

	public void alterarSinistrosAssociados(String placaAntiga, String placaNova, int lidosPorVez) {
		int numeroDeObjetos = lidosPorVez;

		try {
			final FileChannel channel = new RandomAccessFile(f, "rw").getChannel();

			while (numeroDeObjetos != 0) {
				if (getPosicao() * tamanhoDeUmSinistro + tamanhoDeUmSinistro * numeroDeObjetos > f
						.length())
					numeroDeObjetos = (int) contarEntradas() - getPosicao();

				final ByteBuffer bg = channel.map(FileChannel.MapMode.READ_WRITE,
						(getPosicao() * tamanhoDeUmSinistro),
						(tamanhoDeUmSinistro * numeroDeObjetos));

				for (int i = 0; i < numeroDeObjetos; i++) {
					final byte[] lido = new byte[tamanhoPlaca];
					bg.position(i * tamanhoDeUmSinistro + tamanhoCodigo + 1);
					bg.get(lido, 0, tamanhoPlaca);

					final String placaAtual = new String(lido, "ISO8859-1");
					if (placaAtual.equalsIgnoreCase(placaAntiga)) {
						bg.position(i * tamanhoDeUmSinistro + tamanhoCodigo + 1);
						bg.put(placaNova.getBytes(), 0, tamanhoPlaca);
					}
				}
				setPosicao(getPosicao() + numeroDeObjetos);
			}
			channel.close();
		} catch (final FileNotFoundException e1) {
			imprimirStringConvertida("Arquivo nao encontrado!");
		} catch (final EOFException e) {
			imprimirStringConvertida("Fim do arquivo!");
		} catch (final IOException e) {
			imprimirStringConvertida("Voc no tem permisso para acessar este arquivo");
		}

	}

	public TipoSinistro lerSinistroDaPosicao(int posicao) throws ExcecaoPosicaoInvalida {
		return tradutor.traduzirSinistro(super.lerDaPosicao(posicao));
	}

	public int obterProximoCodigo() {
		int proxCod = 0;

		final long numeroDeSinistros = f.length() / tamanhoDeUmSinistro;
		for (int i = 1; i <= numeroDeSinistros; i++) {
			final int codigoAtual = lerSinistroDaPosicao(i).obterCodigo();
			if (codigoAtual > proxCod)
				proxCod = codigoAtual;
		}
		proxCod++;
		return proxCod;
	}

	public int procurarPosicaoDoSinistro(int codigo, int n) {
		setPosicao(0);
		int posicaoEncontrada = 0;
		TiposPesquisa tipoPesquisa = Opcoes.obterTipoPesquisa();

		switch (tipoPesquisa) {
		case SEQUENCIAL:
			posicaoEncontrada = procurarPosicaoDoSinistroSequencial(codigo, n);
			break;
		case INDEXADA:
			posicaoEncontrada = procurarPosicaoDoSinistroIndexado(codigo);
			break;
		}

		return posicaoEncontrada;
	}

	private int procurarPosicaoDoSinistroIndexado(int codigo) {
		int posicao = 0;

		TabelaDeHash<Integer, Integer> t = GerenciadorDeIndices.obterIndiceSinistrosCodigo();
		Integer achado = t.retorne(codigo);
		if (achado != null)
			posicao = (achado);

		return posicao;
	}

	private int procurarPosicaoDoSinistroSequencial(int codigo, int n) {
		int posicaoEncontrada = 0;
		int contador = 0;
		boolean achou = false;

		String[] lidos = new String[n];

		while (!achou && !acabouArquivo()) {
			lidos = lerDoArquivo(n);
			for (int i = 0; i < lidos.length && !achou; i++) {
				contador++;
				if (codigo == Integer.parseInt(lidos[i].substring(0, 6))) {
					achou = true;
					posicaoEncontrada = contador;
				}
			}
		}

		return posicaoEncontrada;
	}

	public TipoSinistro procurarSinistro(int codigo, int n) {
		setPosicao(0);
		TipoSinistro retornado = null;
		int posicao = procurarPosicaoDoSinistro(codigo, n);
		if (posicao != 0)
			retornado = tradutor.traduzirSinistro(lerDaPosicao(posicao));

		return retornado;
	}

	public ListaDuplamenteEncadeada<Sinistro> procurarSinistrosDeUmVeiculo(String placa, int n) {
		TiposPesquisa tipoPesquisa = Opcoes.obterTipoPesquisa();

		setPosicao(0);
		ListaDuplamenteEncadeada<Sinistro> retornados = null;
		switch (tipoPesquisa) {
		case SEQUENCIAL:
			retornados = procurarSinistrosDeUmVeiculoSequencial(placa, n);
			break;
		case INDEXADA:
			retornados = procurarSinistrosDeUmVeiculoIndexado(placa);
			break;
		}

		return retornados;
	}

	private ListaDuplamenteEncadeada<Sinistro> procurarSinistrosDeUmVeiculoIndexado(String placa) {
		final ListaDuplamenteEncadeada<Sinistro> listaDeSinistros = new ListaDuplamenteEncadeada<Sinistro>();

		TabelaDeHash<String, Lista<Integer>> t = GerenciadorDeIndices.obterIndiceSinistrosPlaca();
		Lista<Integer> achados = t.retorne(placa);
		if (achados != null)
			for (Iterador<Integer> iter = achados.retorneIterador(); iter.temMais();)
				// TODO Traduzindo para depois formatar e salvar no -p.txt?
				listaDeSinistros.insiraNoFim(tradutor.traduzirSinistro(lerDaPosicao(iter
						.retorneProximo())));

		return listaDeSinistros;
	}

	private ListaDuplamenteEncadeada<Sinistro> procurarSinistrosDeUmVeiculoSequencial(String placa,
			int n) {
		final ListaDuplamenteEncadeada<Sinistro> listaDeSinistros = new ListaDuplamenteEncadeada<Sinistro>();
		String[] lidos = new String[n];

		while (!acabouArquivo()) {
			lidos = lerDoArquivo(n);
			for (int i = 0; i < lidos.length; i++)
				if (placa.equalsIgnoreCase(lidos[i].substring(7, 14)))
					listaDeSinistros.insiraNoFim(tradutor.traduzirSinistro(lidos[i]));

		}

		return listaDeSinistros;
	}

	public void removerSinistro(int posicaoDoEncontrado) {
		removerLinha(posicaoDoEncontrado);
	}

	public void salvarNaPosicao(int posicao, TipoSinistro sinistro) {
		final String sinistroFormatado = tradutor.formatarSinistro(sinistro);
		super.salvarNaPosicao(posicao, sinistroFormatado);
	}

	public void serializar(TipoSinistro s) {
		final String sinistroFormatado = tradutor.formatarSinistro(s);
		super.serializar(sinistroFormatado);

	}

}
