package ed20081.g1.tp1.persistencia;

import ine5408.th.TabelaDeHash;
import ed20081.g1.tp1.codigo.TipoVeiculo;
import ed20081.g1.tp1.enums.TiposPesquisa;
import ed20081.g1.tp1.excecoes.ExcecaoPosicaoInvalida;
import ed20081.g1.tp1.pesquisa.GerenciadorDeIndices;
import ed20081.g1.tp1.principal.Opcoes;

public class SerializadorVeiculo extends Serializador {

	private static SerializadorVeiculo instanciaUnica;

	public static SerializadorVeiculo obterInstancia() {
		if (instanciaUnica == null)
			instanciaUnica = new SerializadorVeiculo();

		return instanciaUnica;
	}

	private final Tradutor tradutor = Tradutor.obterInstancia();

	private final int tamanhoDeUmVeiculo = tradutor.obterTamanhoVeiculo();

	private SerializadorVeiculo() {
		fixarTamanhoDoSerializado(tamanhoDeUmVeiculo);
	}

	public TipoVeiculo lerVeiculoDaPosicao(int posicao) throws ExcecaoPosicaoInvalida {
		return tradutor.traduzirVeiculo(super.lerDaPosicao(posicao));
	}

	public int procurarPosicaoDoVeiculo(String placa, int n) {
		setPosicao(0);
		int posicaoEncontrada = 0;
		TiposPesquisa tipoPesquisa = Opcoes.obterTipoPesquisa();

		switch (tipoPesquisa) {
		case SEQUENCIAL:
			posicaoEncontrada = procurarPosicaoDoVeiculoSequencial(placa, n);
			break;
		case INDEXADA:
			posicaoEncontrada = procurarPosicaoDoVeiculoIndexada(placa);
			break;
		}

		return posicaoEncontrada;
	}

	private int procurarPosicaoDoVeiculoIndexada(String placa) {
		int posicao = 0;

		TabelaDeHash<String, Integer> t = GerenciadorDeIndices.obterIndiceVeiculos();
		Integer achado = t.retorne(placa);
		if (achado != null)
			posicao = (achado);

		return posicao;
	}

	private int procurarPosicaoDoVeiculoSequencial(String placa, int n) {
		int posicaoEncontrada = 0;
		int contador = 0;
		boolean achou = false;

		String[] lidos = new String[n];

		while (!achou && !acabouArquivo()) {
			lidos = lerDoArquivo(n);
			for (int i = 0; i < lidos.length && !achou; i++) {
				contador++;
				if (placa.equalsIgnoreCase(lidos[i].substring(0, 7))) {
					achou = true;
					posicaoEncontrada = contador;
				}
			}
		}

		return posicaoEncontrada;
	}

	public TipoVeiculo procurarVeiculo(String placa, int numeroDeVeiculos) {
		setPosicao(0);
		TipoVeiculo retornado = null;
		int achado = procurarPosicaoDoVeiculo(placa, numeroDeVeiculos);
		if (achado != 0)
			retornado = tradutor.traduzirVeiculo(lerDaPosicao(achado));
		return retornado;

	}

	public void removerVeiculo(int posicao) {
		removerLinha(posicao);
	}

	public void salvarNaPosicao(int posicao, TipoVeiculo antigo) {
		final String veiculoFormatado = tradutor.formatarVeiculo(antigo);
		super.salvarNaPosicao(posicao, veiculoFormatado);
	}

	public void serializar(TipoVeiculo v) {
		final String veiculoFormatado = tradutor.formatarVeiculo(v);
		super.serializar(veiculoFormatado);
	}

}
