package ed20081.g1.tp1.persistencia;

import static ed20081.g1.tp1.util.StringUtils.fixarTamanhoDeString;

import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import ed20081.g1.tp1.codigo.Sinistro;
import ed20081.g1.tp1.codigo.TipoSinistro;
import ed20081.g1.tp1.codigo.TipoVeiculo;
import ed20081.g1.tp1.codigo.Veiculo;

public class Tradutor {

	private static Tradutor instanciaUnica;

	public static Tradutor obterInstancia() {
		if (instanciaUnica == null)
			instanciaUnica = new Tradutor();

		return instanciaUnica;
	}

	private final String EOL = System.getProperty("line.separator");

	private final int TAM_EOL = EOL.length();
	private final SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
	private final int tamanhoCodigo = 6;

	private final int tamanhoPlaca = 7;
	private final int tamanhoData = 10;
	private final int tamanhoMunicipio = 30;
	private final int tamanhoEstado = 2;
	private final int tamanhoEndereco = 50;
	private final int tamanhoEmbriagado = 3;
	private final int tamanhoPerdaTotal = 3;
	private final int tamanhoVitimasFatais = 4;
	private final int tamanhoVitimasNaoFatais = 4;
	private final int tamanhoDescricao = 200;
	private final String separador = "";
	private final int tamanhoAno = 4;

	private final int tamanhoMarca = 10;
	private final int tamanhoModelo = 20;
	private final int tamanhoCor = 15;
	private final int tamanhoCombustivel = 20;
	private final int tamanhoDeUmVeiculo = tamanhoPlaca + tamanhoAno + tamanhoMarca + tamanhoModelo
			+ tamanhoCor + tamanhoCombustivel + separador.length() * 5 + TAM_EOL;

	private final int tamanhoDeUmSinistro = tamanhoCodigo + tamanhoPlaca + tamanhoData
			+ tamanhoMunicipio + tamanhoEstado + tamanhoEndereco + tamanhoEmbriagado
			+ tamanhoPerdaTotal + tamanhoVitimasFatais + tamanhoVitimasNaoFatais + tamanhoDescricao
			+ separador.length() * 10 + TAM_EOL;

	private Tradutor() {

	}

	public String formatarSinistro(TipoSinistro s) {
		final String placa = s.obterPlaca();
		final String municipio = s.obterMunicipio();
		final String estado = s.obterEstado();
		final String endereco = s.obterEndereco();
		final String descricao = s.obterDescricao();

		final String codigo = new DecimalFormat("000000").format(s.obterCodigo());
		final String data = sdf.format(s.obterData());
		final String perdaTotal = s.perdaTotal() ? "sim" : "no";
		final String motoristaEmbriagado = s.motoristaEmbriagado() ? "sim" : "no";
		final String vitimasFatais = new DecimalFormat("0000").format(s.obterVitimasFatais());
		final String vitimasNaoFatais = new DecimalFormat("0000").format(s.obterVitimasNaoFatais());

		final String sinistroFormatado = codigo + separador + placa + separador + data + separador
				+ fixarTamanhoDeString(municipio, tamanhoMunicipio) + separador
				+ fixarTamanhoDeString(estado, tamanhoEstado) + separador
				+ fixarTamanhoDeString(endereco, tamanhoEndereco) + separador + motoristaEmbriagado
				+ separador + perdaTotal + separador
				+ fixarTamanhoDeString(vitimasNaoFatais, tamanhoVitimasNaoFatais) + separador
				+ fixarTamanhoDeString(vitimasFatais, tamanhoVitimasFatais) + separador
				+ fixarTamanhoDeString(descricao, tamanhoDescricao) + EOL;

		return sinistroFormatado;
	}

	public String formatarVeiculo(TipoVeiculo v) {

		String veiculoFormatado = "";

		final String marca = v.obterMarca();
		final String modelo = v.obterModelo();
		final String cor = v.obterCor();
		final String combustivel = v.obterCombustivel();

		veiculoFormatado += v.obterPlaca() + separador + v.obterAnoDeFabricacao() + separador
				+ fixarTamanhoDeString(marca, tamanhoMarca) + separador
				+ fixarTamanhoDeString(modelo, tamanhoModelo) + separador
				+ fixarTamanhoDeString(cor, tamanhoCor) + separador
				+ fixarTamanhoDeString(combustivel, tamanhoCombustivel) + EOL;

		return veiculoFormatado;
	}

	public int obterTamanhoCodigo() {
		return tamanhoCodigo;
	}

	public int obterTamanhoPlaca() {
		return tamanhoPlaca;
	}

	public int obterTamanhoSinistro() {
		return tamanhoDeUmSinistro;
	}

	public int obterTamanhoVeiculo() {
		return tamanhoDeUmVeiculo;
	}

	public Sinistro traduzirSinistro(String repr) {
		Date data = null;
		final String[] campos = repr.split(separador);

		final int codigo = Integer.parseInt(campos[0]);
		final String placa = campos[1];

		try {
			data = sdf.parse(campos[2]);
		} catch (final ParseException e) {
			System.out.println("Para a data use o formato dd/MM/yyyy");
		}

		final String municipio = campos[3].trim();
		final String estado = campos[4];
		final String endereco = campos[5].trim();
		final boolean motoristaEmbriagado = campos[6].equals("sim");
		final boolean perdaTotal = campos[7].equals("sim");
		final int vitimasFatais = Integer.parseInt(campos[8]);
		final int vitimasNaoFatais = Integer.parseInt(campos[9]);
		final String descricao = campos[10].substring(0, tamanhoDescricao).trim();

		return new Sinistro(codigo, placa, data, municipio, estado, endereco, motoristaEmbriagado,
				perdaTotal, vitimasNaoFatais, vitimasFatais, descricao);
	}

	public TipoVeiculo traduzirVeiculo(String repr) {

		final String[] campos = repr.split(separador);

		final String placa = campos[0];
		final int ano = Integer.valueOf(campos[1]);
		final String marca = campos[2].trim();
		final String modelo = campos[3].trim();
		final String cor = campos[4].trim();
		final String combustivel = campos[5].trim();

		return new Veiculo(placa, ano, marca, modelo, cor, combustivel);
	}
}
