package ed20081.g1.tp1.principal;

import static ed20081.g1.tp1.util.StringUtils.imprimirStringConvertida;
import static ed20081.g1.tp1.util.StringUtils.imprimirTempo;

import java.util.Date;
import java.util.Scanner;

import ed20081.g1.tp1.aleatorios.GeradorAleatorioDeSinistros;
import ed20081.g1.tp1.aleatorios.GeradorAleatorioDeVeiculos;
import ed20081.g1.tp1.codigo.Sinistro;
import ed20081.g1.tp1.codigo.TipoSinistro;
import ed20081.g1.tp1.codigo.TipoVeiculo;
import ed20081.g1.tp1.codigo.Veiculo;
import ed20081.g1.tp1.enums.Modos;
import ed20081.g1.tp1.enums.TiposPesquisa;
import ed20081.g1.tp1.excecoes.ExcecaoPosicaoInvalida;
import ed20081.g1.tp1.persistencia.SerializadorPesquisa;
import ed20081.g1.tp1.persistencia.SerializadorSinistro;
import ed20081.g1.tp1.persistencia.SerializadorVeiculo;
import ed20081.g1.tp1.pesquisa.GerenciadorDeIndices;
import ed20081.g1.tp1.principal.ui.LeitorDados;
import ed20081.g1.tp1.principal.ui.ModificadorDeSinistros;
import ed20081.g1.tp1.principal.ui.ModificadorDeVeiculos;
import ed20081.g1.tp1.util.ListaDuplamenteEncadeada;

public class Main {

	private static SerializadorVeiculo serializadorVeiculo = SerializadorVeiculo.obterInstancia();
	private static SerializadorSinistro serializadorSinistro = SerializadorSinistro
			.obterInstancia();
	private static SerializadorPesquisa serializadorPesquisa = SerializadorPesquisa
			.obterInstancia();

	private static LeitorDados leitor = LeitorDados.obterInstancia();
	private static ModificadorDeSinistros modificador = ModificadorDeSinistros.obterInstancia();
	private static Scanner scanner = new Scanner(System.in);

	// TODO Alterar todos mtodos iterativos para usar while ao invs de chamada
	// recursiva

	private static void alterarDadosDeUmSinistro() {
		final int codigo = leitor.lerCodigo();
		final int posicaoDoEncontrado = serializadorSinistro
				.procurarPosicaoDoSinistro(codigo, 1000);
		if (posicaoDoEncontrado == 0)
			imprimirStringConvertida("Nenhum sinistro com esse cdigo encontrado no arquivo");
		else {
			TipoSinistro antigo = serializadorSinistro.lerSinistroDaPosicao(posicaoDoEncontrado);
			imprimirStringConvertida("Os campos deixados em branco mantero os dados anteriores.");
			antigo = modificador.modificarSinistro(antigo);
			serializadorSinistro.salvarNaPosicao(posicaoDoEncontrado, antigo);
		}

		imprimirStringConvertida("Deseja alterar outro sinistro? (s/N)");
		final String desejo = scanner.nextLine();
		if (desejo.equalsIgnoreCase("s"))
			alterarDadosDeUmSinistro();

	}

	private static void alterarDadosDeUmVeiculo() {
		final String placa = leitor.lerPlaca();

		TipoVeiculo antigo = null;

		final int posicaoDoEncontrado = serializadorVeiculo.procurarPosicaoDoVeiculo(placa, 1000);

		if (posicaoDoEncontrado == 0)
			imprimirStringConvertida("Nenhum veculo com essa placa encontrado no arquivo");
		else {
			antigo = serializadorVeiculo.lerVeiculoDaPosicao(posicaoDoEncontrado);

			imprimirStringConvertida("Os campos deixados em branco mantero os dados anteriores.");
			ModificadorDeVeiculos.obterInstancia().modificarVeiculo(antigo);
			serializadorVeiculo.salvarNaPosicao(posicaoDoEncontrado, antigo);
		}

		imprimirStringConvertida("Deseja alterar outro veculo? (s/N)");
		final String desejo = scanner.nextLine();
		if (desejo.equalsIgnoreCase("s"))
			alterarDadosDeUmVeiculo();
	}

	private static void cadastrarAleatoriamenteSinistros() {
		final double msAntes = System.currentTimeMillis();
		final GeradorAleatorioDeSinistros sorteador = new GeradorAleatorioDeSinistros(
				serializadorSinistro);
		final int n = Opcoes.obterQtdRegsAleatorios();

		imprimirStringConvertida("Cadastrando aleatoriamente " + n + " sinistros");

		try {
			sorteador.cadastrarAleatoriamenteN(n);
		} catch (final ExcecaoPosicaoInvalida e) {
			imprimirStringConvertida("Voc no pode gerar sinistros pois no h nenhum veculo no banco de dados!");
		}
		final double msDepois = System.currentTimeMillis();
		imprimirTempo((msDepois - msAntes) / 1000, "O cadastro");
	}

	private static void cadastrarAleatoriamenteVeiculos() {
		final double msAntes = System.currentTimeMillis();
		final GeradorAleatorioDeVeiculos sorteador = new GeradorAleatorioDeVeiculos(
				serializadorVeiculo);
		final int n = Opcoes.obterQtdRegsAleatorios();

		imprimirStringConvertida("Cadastrando aleatoriamente " + n + " veculos");

		sorteador.cadastrarAleatoriamenteN(n);
		final double msDepois = System.currentTimeMillis();
		imprimirTempo((msDepois - msAntes) / 1000, "O cadastro");
	}

	private static void cadastrarSinistroManualmente() {

		imprimirStringConvertida("(Digite os dados do sinistro a ser cadastrado)");

		String placa = leitor.lerPlaca();
		while (serializadorVeiculo.procurarVeiculo(placa, 1000) == null) {
			imprimirStringConvertida("No foi encontrado veculo com esta placa!");
			placa = leitor.lerPlaca();
		}

		final Date data = leitor.lerData();
		final String municipio = leitor.lerCampoNaoVazio("Municpio: ");
		final String estado = leitor.lerEstado();
		final String endereco = leitor.lerCampoNaoVazio("Endereco: ");
		final boolean motoristaEmbriagado = leitor.lerMotoristaEmbriagado();
		final boolean perdaTotal = leitor.lerPerdaTotal();
		final int vitimasNaoFatais = leitor.lerVitimasNaoFatais();
		final int vitimasFatais = leitor.lerVitimasFatais();
		final String descricao = leitor.lerCampoNaoVazio("Descrio do sinistro: ");

		final int codigo = serializadorSinistro.obterProximoCodigo();

		final TipoSinistro novo = new Sinistro(codigo, placa, data, municipio, estado, endereco,
				motoristaEmbriagado, perdaTotal, vitimasNaoFatais, vitimasFatais, descricao);

		serializadorSinistro.serializar(novo);

		if (Opcoes.obterTipoPesquisa() == TiposPesquisa.INDEXADA)
			GerenciadorDeIndices.adicionarNosIndicesDeSinistro(novo);
		imprimirStringConvertida("Sinistro inserido com sucesso!");

		imprimirStringConvertida("Deseja cadastrar um novo sinistro? (s/N): ");
		final String desejo = scanner.nextLine();

		if (desejo.equalsIgnoreCase("s"))
			cadastrarSinistroManualmente();
	}

	private static void cadastrarVeiculoManualmente() {

		imprimirStringConvertida("(Digite os dados do veculo a ser cadastrado)");

		String placa = leitor.lerPlaca();

		while (serializadorVeiculo.procurarVeiculo(placa, 1000) != null) {
			imprimirStringConvertida("Veculo j est no banco de dados!");
			placa = leitor.lerPlaca();
		}

		final int ano = leitor.lerAno();
		final String marca = leitor.lerCampoNaoVazio("Marca: ");
		final String modelo = leitor.lerCampoNaoVazio("Modelo: ");
		final String cor = leitor.lerCampoNaoVazio("Cor: ");
		final String combustvel = leitor.lerCampoNaoVazio("Combustvel: ");

		final TipoVeiculo novo = new Veiculo(placa, ano, marca, modelo, cor, combustvel);

		serializadorVeiculo.serializar(novo);
		imprimirStringConvertida("Veculo inserido com sucesso!");

		imprimirStringConvertida("Deseja cadastrar um novo veculo? (s/N): ");
		final String desejo = scanner.nextLine();

		if (desejo.equalsIgnoreCase("s"))
			cadastrarVeiculoManualmente();

	}

	public static void main(String[] args) {
		Interpretador.interpretar(args);

		Modos modo = Opcoes.obterModo();
		TiposPesquisa tipoDePesquisa = Opcoes.obterTipoPesquisa();

		imprimirStringConvertida(modo.toString());
		imprimirStringConvertida(tipoDePesquisa.toString());

		serializadorVeiculo.fixarArquivo(Opcoes.obterArquivoVeiculos());
		serializadorSinistro.fixarArquivo(Opcoes.obterArquivoSinistros());
		serializadorPesquisa.fixarArquivo(Opcoes.obterArquivoPesquisa());
		if (tipoDePesquisa == TiposPesquisa.INDEXADA)
			GerenciadorDeIndices.popularIndices();

		switch (modo) {
		case CADASTRO:
			imprimirStringConvertida("Arquivo utilizado: " + Opcoes.obterArquivoVeiculos());
			cadastrarVeiculoManualmente();
			break;
		case CADASTROSINISTRO:
			imprimirStringConvertida("Arquivo utilizado: " + Opcoes.obterArquivoSinistros());
			cadastrarSinistroManualmente();
			break;
		case CADASTROALEATORIOVEICULO:
			imprimirStringConvertida("Arquivo utilizado: " + Opcoes.obterArquivoVeiculos());
			cadastrarAleatoriamenteVeiculos();
			break;
		case PESQUISADEVEICULO:
			imprimirStringConvertida("Arquivo utilizado: " + Opcoes.obterArquivoVeiculos());
			pesquisarVeiculo();
			break;
		case CADASTROALEATORIOSINISTRO:
			imprimirStringConvertida("Arquivo utilizado: " + Opcoes.obterArquivoSinistros());
			cadastrarAleatoriamenteSinistros();
			break;
		case PESQUISADESINISTRO:
			imprimirStringConvertida("Arquivo utilizado: " + Opcoes.obterArquivoSinistros());
			pesquisarSinistro();
			break;
		case PESQUISADESINISTROSDEUMVEICULO:
			imprimirStringConvertida("Arquivo utilizado: " + Opcoes.obterArquivoSinistros());
			pesquisarSinistrosDeUmVeiculo();
			break;
		case ALTERARDADOSDEUMVEICULO:
			imprimirStringConvertida("Arquivo utilizado: " + Opcoes.obterArquivoVeiculos());
			alterarDadosDeUmVeiculo();
			break;
		case ALTERARDADOSDEUMSINISTRO:
			imprimirStringConvertida("Arquivo utilizado: " + Opcoes.obterArquivoSinistros());
			alterarDadosDeUmSinistro();
			break;
		case REMOVERSINISTRO:
			imprimirStringConvertida("Arquivo utilizado: " + Opcoes.obterArquivoSinistros());
			removerSinistro();
			break;
		case REMOVERVEICULO:
			imprimirStringConvertida("Arquivo utilizado: " + Opcoes.obterArquivoVeiculos());
			removerVeiculo();
			break;
		case NUMEROVEICULOS:
			imprimirStringConvertida("Nmero de veculos cadastrados:");
			final long numeroV = obterNumeroDeVeiculos();
			imprimirStringConvertida(String.valueOf(numeroV));
			break;
		case NUMEROSINISTROS:
			imprimirStringConvertida("Nmero de veculos cadastrados:");
			final long numeroS = obterNumeroDeSinitros();
			imprimirStringConvertida(String.valueOf(numeroS));
			break;
		default:
			imprimirStringConvertida("Arquivo utilizado: " + Opcoes.obterArquivoVeiculos());
			pesquisarVeiculo();
			break;
		}

	}

	private static long obterNumeroDeSinitros() {
		return serializadorSinistro.contarEntradas();
	}

	private static long obterNumeroDeVeiculos() {
		return serializadorVeiculo.contarEntradas();
	}

	private static void pesquisarSinistro() {

		final int codigo = leitor.lerCodigo();
		final double msAntes = System.currentTimeMillis();

		final TipoSinistro encontrado = serializadorSinistro.procurarSinistro(codigo, 1000);
		final double msDepois = System.currentTimeMillis();

		if (encontrado == null) {
			imprimirStringConvertida("Nenhum sinistro com esse cdigo foi encontrado no arquivo.");
			imprimirTempo((msDepois - msAntes) / 1000, "A pesquisa");
		} else {
			imprimirStringConvertida("Foi encontrado o sinistro com o cdigo " + codigo);
			imprimirTempo((msDepois - msAntes) / 1000, "A pesquisa");
			serializarSinistroEncontrado(encontrado);
		}
		imprimirStringConvertida("Deseja pesquisar novamente? (s/N): ");
		final String desejo = scanner.nextLine();
		if (desejo.equalsIgnoreCase("s"))
			pesquisarSinistro();

	}

	private static void pesquisarSinistrosDeUmVeiculo() {
		final String placa = leitor.lerPlaca();

		final ListaDuplamenteEncadeada<Sinistro> encontrados = serializadorSinistro
				.procurarSinistrosDeUmVeiculo(placa, 1000);

		if (encontrados.estaVazia())
			imprimirStringConvertida("Nenhum sinistro relacionado  placa informada foi encontrado no arquivo.");
		else {
			imprimirStringConvertida("Foram encontrados sinistros relacionados  placa " + placa);
			serializarSinistrosEncontrados(encontrados);
			imprimirStringConvertida("Nmero de sinistros relacionados: "
					+ encontrados.retorneTamanho());
		}
		imprimirStringConvertida("Deseja pesquisar novamente? (s/N): ");
		final String desejo = scanner.nextLine();
		if (desejo.equalsIgnoreCase("s"))
			pesquisarSinistrosDeUmVeiculo();
	}

	private static void pesquisarVeiculo() {
		final String placa = leitor.lerPlaca();
		final double msAntes = System.currentTimeMillis();

		final TipoVeiculo encontrado = serializadorVeiculo.procurarVeiculo(placa, 1000);
		final double msDepois = System.currentTimeMillis();

		if (encontrado == null) {

			imprimirStringConvertida("Nenhum veculo com essa placa encontrado no arquivo");
			imprimirTempo((msDepois - msAntes) / 1000, "A pesquisa");
		} else {
			imprimirStringConvertida("Veculo encontrado com a placa " + placa);
			imprimirTempo((msDepois - msAntes) / 1000, "A pesquisa");
			serializarVeiculoEncontrado(encontrado);

		}

		imprimirStringConvertida("Deseja pesquisar novamente? (s/N): ");

		final String desejo = scanner.nextLine();
		if (desejo.equalsIgnoreCase("s"))
			pesquisarVeiculo();

	}

	private static void removerSinistro() {
		final int codigo = leitor.lerCodigo();
		TipoSinistro antigo = null;

		final int posicaoDoEncontrado = serializadorSinistro
				.procurarPosicaoDoSinistro(codigo, 1000);

		if (posicaoDoEncontrado == 0)
			imprimirStringConvertida("Nenhum sinistro com esse cdigo encontrado no arquivo");
		else {
			antigo = serializadorSinistro.lerSinistroDaPosicao(posicaoDoEncontrado);
			System.out.println(antigo.toString());

			imprimirStringConvertida("Tem certeza que deseja remover o sinistro? (s/N)");
			final String desejo = scanner.nextLine();
			if (desejo.equalsIgnoreCase("s"))
				serializadorSinistro.removerSinistro(posicaoDoEncontrado);
		}
		imprimirStringConvertida("Deseja remover algum outro sinistro? (s/N)");
		final String desejo2 = scanner.nextLine();
		if (desejo2.equalsIgnoreCase("s"))
			removerSinistro();

	}

	private static void removerVeiculo() {
		final String placa = leitor.lerPlaca();
		TipoVeiculo encontrado = null;

		final int posicaoDoEncontrado = serializadorVeiculo.procurarPosicaoDoVeiculo(placa, 1000);

		if (posicaoDoEncontrado == 0)
			imprimirStringConvertida("Nenhum veculo com essa placa encontrado no arquivo");
		else {
			encontrado = serializadorVeiculo.lerVeiculoDaPosicao(posicaoDoEncontrado);
			imprimirStringConvertida(encontrado.toString());

			imprimirStringConvertida("Tem certeza que deseja remover o veculo? (s/N)");
			final String desejo = scanner.nextLine();

			if (desejo.equalsIgnoreCase("s")) {
				final ListaDuplamenteEncadeada<Sinistro> sinistrosRelacionados = serializadorSinistro
						.procurarSinistrosDeUmVeiculo(placa, 1000);
				int numeroDeSinistros = sinistrosRelacionados.retorneTamanho();
				if (numeroDeSinistros == 0)
					serializadorVeiculo.removerVeiculo(posicaoDoEncontrado);
				else
					imprimirStringConvertida("Existem " + numeroDeSinistros
							+ " sinistros relacionados a este veculo!");

			}
		}

		imprimirStringConvertida("Deseja remover algum outro veculo? (s/N)");
		final String desejo2 = scanner.nextLine();
		if (desejo2.equalsIgnoreCase("s"))
			removerVeiculo();
	}

	private static void serializarSinistroEncontrado(TipoSinistro encontrado) {

		serializadorPesquisa.serializarSinistro(encontrado);

	}

	private static void serializarSinistrosEncontrados(
			ListaDuplamenteEncadeada<Sinistro> encontrados) {

		serializadorPesquisa.serializarSinistros(encontrados);

	}

	private static void serializarVeiculoEncontrado(TipoVeiculo encontrado) {

		serializadorPesquisa.serializarVeiculo(encontrado);

	}

}
