package ed20081.g1.tp1.principal.ui;

import static ed20081.g1.tp1.util.StringUtils.imprimirStringConvertida;

import java.util.Date;

import ed20081.g1.tp1.excecoes.CampoVazio;
import ed20081.g1.tp1.excecoes.ExcecaoCampoInvalido;
import ed20081.g1.tp1.validacao.ValidadorDeSinistros;
import ed20081.g1.tp1.validacao.ValidadorDeVeiculos;

public class LeitorDados extends Leitor {

	private static LeitorDados instanciaUnica;

	public static LeitorDados obterInstancia() {
		if (instanciaUnica == null)
			instanciaUnica = new LeitorDados();
		return instanciaUnica;
	}

	private final ValidadorDeVeiculos validadorVeiculo;
	private final ValidadorDeSinistros validadorSinistro;

	private LeitorDados() {
		super();
		validadorVeiculo = ValidadorDeVeiculos.obterInstancia();
		validadorSinistro = ValidadorDeSinistros.obterInstancia();
	}

	public int lerAno() {
		int ano = 0;

		try {
			imprimirStringConvertida("Ano (4 dgitos): ");
			ano = Integer.valueOf(scanner.nextLine());
			validadorVeiculo.validarAno(ano);
		} catch (final ExcecaoCampoInvalido e) {
			imprimirStringConvertida(e.getMessage());
			ano = lerAno();
		} catch (final NumberFormatException e) {
			imprimirStringConvertida("O valor digitado no  um nmero!");
			ano = lerAno();
		}

		return ano;
	}

	public String lerCampoNaoVazio(String msg) {
		String campo;
		imprimirStringConvertida(msg);
		campo = scanner.nextLine();
		try {
			validadorVeiculo.validarCampoNaoVazio(campo);
		} catch (final CampoVazio e) {
			e.getMessage();
			campo = lerCampoNaoVazio(msg);
		}
		return campo;
	}

	public int lerCodigo() {
		int codigo = 0;
		try {
			imprimirStringConvertida("Digite o cdigo do sinistro: ");
			codigo = Integer.valueOf(scanner.nextLine());
		} catch (final NumberFormatException e) {
			imprimirStringConvertida("O valor digitado no  um nmero!");
			codigo = lerCodigo();
		}
		return codigo;
	}

	public Date lerData() {
		Date data = null;
		try {
			final String dataString = lerCampoNaoVazio("Data de ocorrncia do sinistro (dd/mm/aaaa): ");
			data = validadorSinistro.validarData(dataString);
		} catch (final ExcecaoCampoInvalido e) {
			imprimirStringConvertida(e.getMessage());
			data = lerData();
		}
		return data;
	}

	public String lerEstado() {
		String estado = lerCampoNaoVazio("Estado (sigla): ");

		try {
			validadorSinistro.validarEstado(estado);
		} catch (final ExcecaoCampoInvalido e) {
			imprimirStringConvertida(e.getMessage());
			estado = lerEstado();
		}
		return estado;
	}

	public boolean lerMotoristaEmbriagado() {
		String embriagado;
		boolean resposta = false;

		imprimirStringConvertida("Motorista embriagado? (s/N): ");
		embriagado = scanner.nextLine();
		try {
			validadorVeiculo.validarCampoNaoVazio(embriagado);
		} catch (final CampoVazio e) {
			imprimirStringConvertida(e.getMessage());
			resposta = lerMotoristaEmbriagado();
		}

		if (embriagado.equalsIgnoreCase("s"))
			resposta = true;

		return resposta;
	}

	public boolean lerPerdaTotal() {
		String perdaTotal;
		boolean resposta = false;

		imprimirStringConvertida("Perda total (s/N): ");
		perdaTotal = scanner.nextLine();
		try {
			validadorVeiculo.validarCampoNaoVazio(perdaTotal);
		} catch (final CampoVazio e) {
			imprimirStringConvertida(e.getMessage());
			resposta = lerPerdaTotal();
		}

		if (perdaTotal.equalsIgnoreCase("s"))
			resposta = true;

		return resposta;
	}

	public String lerPlaca() {
		String placaNaoVazia = lerCampoNaoVazio("Digite a placa: ");

		try {
			validadorVeiculo.validarPlaca(placaNaoVazia);
		} catch (final ExcecaoCampoInvalido e) {
			imprimirStringConvertida(e.getMessage());
			placaNaoVazia = lerPlaca();
		}
		return placaNaoVazia;
	}

	public int lerVitimasFatais() {
		int vitimas = 0;
		try {
			imprimirStringConvertida("Nmero de vtimas fatais: ");
			vitimas = Integer.valueOf(scanner.nextLine());
			validadorSinistro.validarVitimas(vitimas);
		} catch (final ExcecaoCampoInvalido e) {
			imprimirStringConvertida(e.getMessage());
			vitimas = lerVitimasFatais();
		} catch (final NumberFormatException e) {
			imprimirStringConvertida("O valor digitado no  um nmero!");
			vitimas = lerVitimasFatais();
		}
		return vitimas;
	}

	public int lerVitimasNaoFatais() {
		int vitimas = 0;
		try {
			imprimirStringConvertida("Nmero de vtimas no fatais: ");
			vitimas = Integer.valueOf(scanner.nextLine());
			validadorSinistro.validarVitimas(vitimas);
		} catch (final ExcecaoCampoInvalido e) {
			imprimirStringConvertida(e.getMessage());
			vitimas = lerVitimasNaoFatais();
		} catch (final NumberFormatException e) {
			imprimirStringConvertida("O valor digitado no  um nmero!");
			vitimas = lerVitimasNaoFatais();
		}
		return vitimas;
	}

}
