package ed20081.g1.tp1.principal.ui;

import static ed20081.g1.tp1.util.StringUtils.imprimirStringConvertida;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Scanner;

import ed20081.g1.tp1.codigo.TipoSinistro;
import ed20081.g1.tp1.excecoes.ExcecaoCampoInvalido;
import ed20081.g1.tp1.persistencia.SerializadorVeiculo;
import ed20081.g1.tp1.validacao.ValidadorDeSinistros;

public class ModificadorDeSinistros {

	private static ModificadorDeSinistros instanciaUnica;

	public static ModificadorDeSinistros obterInstancia() {
		if (instanciaUnica == null)
			instanciaUnica = new ModificadorDeSinistros();
		return instanciaUnica;
	}

	private final Scanner s = new Scanner(System.in);

	private final ValidadorDeSinistros validadorSinistros = ValidadorDeSinistros.obterInstancia();

	private void modificarData(TipoSinistro antigo) {
		final SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		final String dataHumana = sdf.format(antigo.obterData());
		imprimirStringConvertida("Data antiga: " + dataHumana + "\n Nova data: (dd/MM/aaaa)");

		final String data = s.nextLine();
		if (!data.equals(""))
			try {
				final Date d = validadorSinistros.validarData(data);
				antigo.fixarData(d);
			} catch (final ExcecaoCampoInvalido e) {
				imprimirStringConvertida(e.getMessage());
				modificarData(antigo);
			}
	}

	private void modificarDescricao(TipoSinistro antigo) {
		imprimirStringConvertida("Descrio antiga: " + antigo.obterDescricao()
				+ "\n Nova descrio:");
		final String descricao = s.nextLine();
		if (!descricao.equals(""))
			antigo.fixarDescricao(descricao);

	}

	private void modificarEmbriagado(TipoSinistro antigo) {
		String msg = antigo.motoristaEmbriagado() ? "" : "NO";

		imprimirStringConvertida("Motorista antigo = " + msg
				+ " embriagado \n Novo embriagado? (s/N)");

		final String resposta = s.nextLine();
		if (!resposta.equals(""))
			if (resposta.equalsIgnoreCase("s"))
				antigo.fixarMotoristaEmbriagado(true);
			else
				antigo.fixarMotoristaEmbriagado(false);
	}

	private void modificarEndereco(TipoSinistro antigo) {
		imprimirStringConvertida("Endereo antigo: " + antigo.obterEndereco() + "\n Novo endereo:");
		final String end = s.nextLine();
		if (!end.equals(""))
			antigo.fixarEndereco(end);

	}

	private void modificarEstado(TipoSinistro antigo) {
		imprimirStringConvertida("Estado antigo: " + antigo.obterEstado() + "\n Novo estado:");

		final String estado = s.nextLine();
		if (!estado.equals(""))
			try {
				validadorSinistros.validarEstado(estado);
				antigo.fixarEstado(estado);
			} catch (final ExcecaoCampoInvalido e) {
				imprimirStringConvertida(e.getMessage());
				modificarEstado(antigo);
			}
	}

	private void modificarMunicipio(TipoSinistro antigo) {
		imprimirStringConvertida("Municpio antigo: " + antigo.obterMunicipio()
				+ "\n Novo municpio:");

		final String municipio = s.nextLine();
		if (!municipio.equals(""))
			antigo.fixarMunicipio(municipio);
	}

	private void modificarPerdaTotal(TipoSinistro antigo) {
		String msg = antigo.perdaTotal() ? "sim" : "no";

		imprimirStringConvertida("Perda total antiga = " + msg + " \n Nova perda total: (s/N)");

		final String perda = s.nextLine();
		if (!perda.equals(""))
			if (perda.equalsIgnoreCase("s"))
				antigo.fixarPerdaTotal(true);
			else
				antigo.fixarPerdaTotal(false);
	}

	private void modificarPlaca(TipoSinistro antigo) {
		imprimirStringConvertida("Placa antiga: " + antigo.obterPlaca() + "\n Nova placa:");
		final String placa = s.nextLine();

		if (!placa.equals(""))
			try {
				validadorSinistros.validarPlaca(placa);
				final SerializadorVeiculo serializadorVeiculo = SerializadorVeiculo
						.obterInstancia();

				if (serializadorVeiculo.procurarVeiculo(placa, 1000) == null) {
					imprimirStringConvertida("Veculo no existe no banco de dados!");
					modificarPlaca(antigo);
				} else
					antigo.fixarPlaca(placa);

			} catch (final ExcecaoCampoInvalido e) {
				imprimirStringConvertida(e.getMessage());
				modificarPlaca(antigo);
			}
	}

	public TipoSinistro modificarSinistro(TipoSinistro antigo) {
		modificarPlaca(antigo);
		modificarData(antigo);
		modificarMunicipio(antigo);
		modificarEstado(antigo);
		modificarEndereco(antigo);
		modificarEmbriagado(antigo);
		modificarPerdaTotal(antigo);
		modificarVitimasFatais(antigo);
		modificarVitimasNaoFatais(antigo);
		modificarDescricao(antigo);
		return antigo;

	}

	private void modificarVitimasFatais(TipoSinistro antigo) {
		imprimirStringConvertida("Nmero de vtimas fatais antigo: " + antigo.obterVitimasFatais()
				+ "\n Novo nmero de vtimas fatais:");
		int vitimas;
		final String lido = s.nextLine();
		if (!lido.equals(""))
			try {
				vitimas = Integer.valueOf(lido);
				validadorSinistros.validarVitimas(vitimas);
				antigo.fixarVitimasFatais(vitimas);
			} catch (final ExcecaoCampoInvalido e) {
				imprimirStringConvertida(e.getMessage());
				modificarVitimasFatais(antigo);
			} catch (final NumberFormatException e) {
				imprimirStringConvertida("O valor digitado no  um nmero!");
				modificarVitimasFatais(antigo);
			}

	}

	private void modificarVitimasNaoFatais(TipoSinistro antigo) {
		imprimirStringConvertida("Nmero de vtimas no fatais antigo: "
				+ antigo.obterVitimasNaoFatais() + "\n Novo nmero de vtimas no fatais");
		int vitimas;
		final String lido = s.nextLine();

		if (!lido.equals(""))
			try {
				vitimas = Integer.valueOf(lido);
				validadorSinistros.validarVitimas(vitimas);
				antigo.fixarVitimasNaoFatais(vitimas);
			} catch (final ExcecaoCampoInvalido e) {
				imprimirStringConvertida(e.getMessage());
				modificarVitimasNaoFatais(antigo);
			} catch (final NumberFormatException e) {
				imprimirStringConvertida("O valor digitado no  um nmero!");
				modificarVitimasNaoFatais(antigo);
			}

	}
}
