package ed20081.g1.tp1.testes.sinistro;

import static org.junit.Assert.assertEquals;

import java.util.Calendar;
import java.util.Date;

import org.junit.Before;
import org.junit.Test;

import ed20081.g1.tp1.aleatorios.RandomUtils;
import ed20081.g1.tp1.enums.aleatorios.Estados;
import ed20081.g1.tp1.excecoes.sinistros.ExcecaoDataInvalida;
import ed20081.g1.tp1.excecoes.sinistros.ExcecaoEstadoInvalido;
import ed20081.g1.tp1.excecoes.sinistros.ExcecaoNumDeVitimasInvalido;
import ed20081.g1.tp1.testes.TestesDeValidador;
import ed20081.g1.tp1.validacao.ValidadorDeSinistros;

public class TestesDeValidadorSinistro extends TestesDeValidador {

	private ValidadorDeSinistros validadorDeSinistros;
	private Calendar c;

	@Before
	public void prepararTestes() {
		validador = validadorDeSinistros = ValidadorDeSinistros.obterInstancia();
		c = Calendar.getInstance();
	}

	@Test
	public void qualquerEstadoBrasileiroEhValido() throws ExcecaoEstadoInvalido {
		final Estados estadoBrasileiro = (Estados) RandomUtils.escolherElemento(Estados.values());

		validadorDeSinistros.validarEstado(estadoBrasileiro.toString());
	}

	@Test(expected = ExcecaoDataInvalida.class)
	public void umaDataAnteriorA1oDeJaneiroDe1885EhInvalida() throws ExcecaoDataInvalida {
		validadorDeSinistros.validarData("31/12/1884");
	}

	@Test(expected = ExcecaoDataInvalida.class)
	public void umaDataForaDoFormatoddMMyyyyEhInvalida() throws ExcecaoDataInvalida {
		validadorDeSinistros.validarData("1/3/99");
	}

	@SuppressWarnings("deprecation")
	@Test
	public void umaDataNoFormatoddMMyyyyEntre1De1De1885AteHojeEhValida() throws ExcecaoDataInvalida {
		final Date dataValida = new Date("01/01/2000");

		assertEquals(dataValida, validadorDeSinistros.validarData("01/01/2000"));
	}

	@Test(expected = ExcecaoDataInvalida.class)
	public void umaDataNoFuturoEhInvalida() throws ExcecaoDataInvalida {
		validadorDeSinistros.validarData("01/01/" + (c.get(Calendar.YEAR) + 1));
	}

	@Test(expected = ExcecaoEstadoInvalido.class)
	public void umEstadoNaoBrasileiroEhInvalido() throws ExcecaoEstadoInvalido {
		validadorDeSinistros.validarEstado("NY");
	}

	@Test(expected = ExcecaoNumDeVitimasInvalido.class)
	public void umNumeroDeVitimasNegativoEhInvalido() {
		validadorDeSinistros.validarVitimas(-1);
	}
}
