package ed20081.g1.tp1.testes.sinistro;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import ed20081.g1.tp1.codigo.Sinistro;
import ed20081.g1.tp1.codigo.TipoSinistro;
import ed20081.g1.tp1.persistencia.SerializadorSinistro;
import ed20081.g1.tp1.persistencia.Tradutor;

public class TestesSerializadorSinistro {

	private SerializadorSinistro serial;
	private File arquivoDeTestes;
	private SimpleDateFormat sdf;
	private Tradutor tradutor;

	@After
	public void limparArquivos() {
		arquivoDeTestes.delete();
	}

	@Before
	public void prepararTestes() {
		arquivoDeTestes = new File("./testesSinistros.txt");
		serial = SerializadorSinistro.obterInstancia();
		serial.fixarArquivo(arquivoDeTestes);

		sdf = new SimpleDateFormat("dd/MM/yyyy");
		tradutor = Tradutor.obterInstancia();
	}

	@Test
	public void testeLerDoArquivo() throws ParseException {
		final TipoSinistro s = new Sinistro(12, "MEU1234", sdf.parse("12/12/2000"), "Gramado",
				"RS", "Rua A - 143", false, true, 0, 3, "Bateu de frente com caminho");

		serial.serializar(s);
		final String stringLida = serial.lerDoArquivo(1)[0];

		assertEquals(tradutor.formatarSinistro(s), stringLida);
	}

	@Test
	public void testeProcurarSinistroPeloCodigo() throws ParseException {
		final TipoSinistro s1 = new Sinistro(12, "MEU1234", sdf.parse("12/12/2000"), "Gramado",
				"RS", "Rua A - 143", false, true, 0, 3, "Bateu de frente com caminho");
		final TipoSinistro s2 = new Sinistro(13, "MEU1234", sdf.parse("12/12/2000"), "Canela",
				"RS", "Rua A - 143", true, true, 1, 2, "Bateu de frente com caminho");
		final TipoSinistro s3 = new Sinistro(14, "MEU1234", sdf.parse("12/12/2000"),
				"Porto Alegre", "RS", "Rua A - 143", false, true, 0, 0,
				"Bateu de frente com caminho");
		final TipoSinistro s4 = new Sinistro(15, "MEU1234", sdf.parse("12/12/2000"), "Chapec",
				"SC", "Rua A - 143", true, true, 0, 1, "Bateu de frente com caminho");
		final TipoSinistro s5 = new Sinistro(16, "MEU1234", sdf.parse("12/12/2000"), "Joinville",
				"SC", "Rua A - 143", false, true, 2, 3, "Bateu de frente com caminho");
		final TipoSinistro s6 = new Sinistro(17, "MEU1234", sdf.parse("12/12/2000"), "Gramado",
				"RS", "Rua A - 143", false, true, 1, 3, "Bateu de frente com caminho");

		serial.serializar(s3);
		serial.serializar(s1);
		serial.serializar(s4);
		serial.serializar(s6);
		serial.serializar(s2);
		serial.serializar(s5);

		final TipoSinistro encontrado = serial.procurarSinistro(15, 4);

		assertEquals(s4, encontrado);
	}

	@Test
	public void testeTraduzirSinistro() throws ParseException {
		final String repr = "000016CDF101012/10/2000Chapec                       SCRua B - 173                                       simsim00020001Bateu de frente com caminho                                                                                                                                                                            "
				+ System.getProperty("line.separator");

		final TipoSinistro esperado = new Sinistro(16, "CDF1010", sdf.parse("12/10/2000"),
				"Chapec", "SC", "Rua B - 173", true, true, 2, 1, "Bateu de frente com caminho");

		assertEquals(esperado, tradutor.traduzirSinistro(repr));
	}

}
