package ed20081.g1.tp1.testes.veiculo;

import static org.junit.Assert.assertEquals;

import java.io.File;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import ed20081.g1.tp1.codigo.TipoVeiculo;
import ed20081.g1.tp1.codigo.Veiculo;
import ed20081.g1.tp1.persistencia.SerializadorVeiculo;
import ed20081.g1.tp1.persistencia.Tradutor;

public class TestesSerializadorVeiculo {

	private SerializadorVeiculo serial;
	private File arquivoDeTestes;
	private Tradutor tradutor;

	@After
	public void limparArquivos() {
		arquivoDeTestes.delete();
	}

	@Before
	public void prepararTestes() {
		arquivoDeTestes = new File("./testesVeiculos.txt");
		serial = SerializadorVeiculo.obterInstancia();
		serial.fixarArquivo(arquivoDeTestes);
		tradutor = Tradutor.obterInstancia();
	}

	@Test
	public void testeLerDoArquivo() {
		final TipoVeiculo veiculo = new Veiculo("WTF6666", 1966, "GM", "Hummer", "Laranja",
				"Querosene");

		serial.serializar(veiculo);
		final String stringLida = serial.lerDoArquivo(1)[0];

		assertEquals(tradutor.formatarVeiculo(veiculo), stringLida);
	}

	@Test
	public void testeProcurarVeiculoPelaPlaca() {
		final TipoVeiculo v1 = new Veiculo("ABC1234", 1990, "Ford", "Taurus", "Preto", "Gasolina");
		final TipoVeiculo v2 = new Veiculo("ABC1534", 1990, "Ford", "Golf", "Preto", "Gasolina");
		final TipoVeiculo v4 = new Veiculo("CDF1534", 1980, "Ford", "Ka", "Prata", "Gas");
		final TipoVeiculo v5 = new Veiculo("ABC0000", 1996, "Ford", "Taurus", "Rosa Pink", "Alcool");
		final TipoVeiculo v6 = new Veiculo("ABC1534", 1990, "Fiat", "Hummer", "Invisivel",
				"Gasolina");

		serial.serializar(v4);
		serial.serializar(v2);
		serial.serializar(v1);
		serial.serializar(v6);
		serial.serializar(v5);

		final TipoVeiculo encontrado = serial.procurarVeiculo("ABC0000", 4);

		assertEquals(v5, encontrado);
	}

	@Test
	public void testeTraduzirVeiculo() {
		final String repr = "WTF66661966GM        Hummer              Laranja        Querosene           "
				+ System.getProperty("line.separator");

		final TipoVeiculo esperado = new Veiculo("WTF6666", 1966, "GM", "Hummer", "Laranja",
				"Querosene");

		assertEquals(esperado, tradutor.traduzirVeiculo(repr));
	}

}
