package ed20081.g1.tp1.util;

import ine5408.excecoes.ExcecaoEstruturaCheia;
import ine5408.excecoes.ExcecaoEstruturaVazia;
import ine5408.excecoes.ExcecaoOperacaoIlegal;
import ine5408.excecoes.ExcecaoPosicaoIlegal;
import ine5408.lineares.Lista;
import ine5408.pp.Iterador;
import ine5408.pp.Visitante;

public class ListaDuplamenteEncadeada<T> implements Lista<T> {

	@SuppressWarnings("hiding")
	private class Elemento<T> {
		private T dado;
		private Elemento<T> prximo;
		private Elemento<T> anterior;

		public Elemento(T dado, Elemento<T> prximo, Elemento<T> anterior) {
			this.dado = dado;
			this.prximo = prximo;
			this.anterior = anterior;
		}

		public Elemento<T> getAnterior() {
			return anterior;
		}

		public T getDado() {
			return dado;
		}

		public Elemento<T> getPrximo() {
			return prximo;
		}

		public void setAnterior(Elemento<T> anterior) {
			this.anterior = anterior;
		}

		public void setDado(T dado) {
			this.dado = dado;
		}

		public void setPrximo(Elemento<T> prximo) {
			this.prximo = prximo;
		}

	}

	private class IteradorDuplamenteEncadeado<A> implements Iterador<T> {

		private Elemento<T> atual;

		public IteradorDuplamenteEncadeado() {
			atual = primeiro;
		}

		public T retorneProximo() throws ExcecaoOperacaoIlegal {
			if (!temMais())
				throw new ExcecaoOperacaoIlegal();
			final Elemento<T> retornado = atual;
			atual = atual.getPrximo();
			return retornado.dado;
		}

		public boolean temMais() {
			return atual != null;
		}

	}

	private Elemento<T> primeiro;
	private Elemento<T> ltimo;
	private int tamanho = 0;

	private Elemento<T> encontrarElementoNaPosicao(int pos) {
		int posAtual;
		Elemento<T> atual;
		if (tamanho - pos < tamanho / 2) {
			posAtual = tamanho;
			atual = ltimo;
			while (posAtual > pos) {
				posAtual--;
				atual = atual.getAnterior();
			}

		} else {
			posAtual = 1;
			atual = primeiro;
			while (posAtual < pos) {
				posAtual++;
				atual = atual.getPrximo();
			}
		}
		return atual;
	}

	@SuppressWarnings("unchecked")
	public boolean equals(Object outro) {
		boolean retorno = false;
		if (outro instanceof ListaDuplamenteEncadeada) {
			final ListaDuplamenteEncadeada<T> outraLista = (ListaDuplamenteEncadeada<T>) outro;
			if (this.retorneTamanho() == outraLista.retorneTamanho()) {

				final Iterador<T> iterDessa = this.retorneIterador();
				final Iterador<T> iterOutra = outraLista.retorneIterador();

				retorno = true;
				while (retorno == true && iterDessa.temMais())
					if (!iterDessa.retorneProximo().equals(iterOutra.retorneProximo()))
						retorno = false;
			}
		}
		return retorno;
	}

	public boolean estaCheia() {
		return false;
	}

	public boolean estaVazia() {
		return tamanho == 0;
	}

	public void esvazie() {
		tamanho = 0;
		primeiro = ltimo = null;
	}

	public void insiraNaPosicao(T dado, int posicao) throws ExcecaoEstruturaCheia,
			ExcecaoPosicaoIlegal {
		if (posicao > tamanho + 1 || posicao < 1)
			throw new ExcecaoPosicaoIlegal(posicao);

		if (posicao == 1)
			insiraNoInicio(dado);
		else if (posicao == tamanho + 1)
			insiraNoFim(dado);
		else {
			final Elemento<T> elementoNaPosicao = encontrarElementoNaPosicao(posicao);
			final Elemento<T> elementoAnterior = elementoNaPosicao.getAnterior();

			final Elemento<T> novo = new Elemento<T>(dado, elementoNaPosicao, elementoAnterior);
			elementoAnterior.setPrximo(novo);
			elementoNaPosicao.setAnterior(novo);
			tamanho++;
		}

	}

	public void insiraNoFim(T dado) throws ExcecaoEstruturaCheia {
		if (estaVazia())
			primeiro = ltimo = new Elemento<T>(dado, null, null);
		else {
			final Elemento<T> novo = new Elemento<T>(dado, null, ltimo);
			ltimo.setPrximo(novo);
			ltimo = novo;
		}
		tamanho++;
	}

	public void insiraNoInicio(T arg0) throws ExcecaoEstruturaCheia {
		if (estaVazia())
			primeiro = ltimo = new Elemento<T>(arg0, null, null);
		else {
			final Elemento<T> novo = new Elemento<T>(arg0, primeiro, null);
			primeiro.setAnterior(novo);
			primeiro = novo;
		}
		tamanho++;
	}

	public boolean pertence(T dado) {
		boolean retorno = false;
		final Iterador<T> iter = retorneIterador();

		while (iter.temMais() && retorno == false) {
			final T dadoAtual = iter.retorneProximo();
			if (dadoAtual == dado)
				retorno = true;
		}

		return retorno;
	}

	public void recebaVisitante(Visitante<T> visitante) {
		final Iterador<T> iter = retorneIterador();

		while (!visitante.satisfeito() && iter.temMais())
			visitante.visite(iter.retorneProximo());
	}

	public T removaDaPosicao(int posicao) throws ExcecaoEstruturaVazia, ExcecaoPosicaoIlegal {

		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		if (posicao > tamanho || posicao < 1)
			throw new ExcecaoPosicaoIlegal(posicao);

		final Elemento<T> elementoNaPosicao = encontrarElementoNaPosicao(posicao);

		removerElementoAtual(elementoNaPosicao);

		return elementoNaPosicao.getDado();
	}

	public T removaDoFim() throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		final T ultimoDado = ltimo.getDado();

		if (tamanho == 1)
			this.esvazie();
		else {
			final Elemento<T> penultimo = ltimo.getAnterior();
			penultimo.setPrximo(null);
			ltimo = penultimo;
			tamanho--;
		}

		return ultimoDado;
	}

	public T removaDoInicio() throws ExcecaoEstruturaVazia {
		T dadoDoInicio;
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		dadoDoInicio = primeiro.getDado();

		if (tamanho == 1)
			esvazie();
		else {
			final Elemento<T> segundo = primeiro.getPrximo();
			segundo.setAnterior(null);
			primeiro = segundo;
			tamanho--;
		}

		return dadoDoInicio;
	}

	public Lista<T> removaIguais(T dado) throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		final Lista<T> listaDeIguais = new ListaDuplamenteEncadeada<T>();

		Elemento<T> atual = primeiro;
		while (atual != null) {
			if (atual.getDado().equals(dado)) {
				listaDeIguais.insiraNoInicio(dado);
				removerElementoAtual(atual);
			}
			atual = atual.getPrximo();
		}
		return listaDeIguais;
	}

	public T removaIgual(T dado) throws ExcecaoEstruturaVazia {

		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		T removido = null;
		Elemento<T> atual = primeiro;
		while (removido == null && atual != null) {
			if (atual.getDado().equals(dado)) {
				removido = atual.getDado();
				removerElementoAtual(atual);
			}
			atual = atual.getPrximo();
		}
		return removido;
	}

	private void removerElementoAtual(final Elemento<T> atual) {
		final Elemento<T> elementoAnterior = atual.getAnterior();
		final Elemento<T> elementoPosterior = atual.getPrximo();

		if (elementoAnterior != null)
			elementoAnterior.setPrximo(elementoPosterior);
		else
			primeiro = elementoPosterior;

		if (elementoPosterior != null)
			elementoPosterior.setAnterior(elementoAnterior);
		else
			ltimo = elementoAnterior;
		tamanho--;
	}

	public T retorneDaPosicao(int pos) throws ExcecaoEstruturaVazia, ExcecaoPosicaoIlegal {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		if (pos < 1 || pos > tamanho)
			throw new ExcecaoPosicaoIlegal(pos);

		return encontrarElementoNaPosicao(pos).getDado();
	}

	public T retorneDoFim() throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		return ltimo.getDado();
	}

	public T retorneDoInicio() throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		return primeiro.getDado();
	}

	public Iterador<T> retorneIterador() {
		return new IteradorDuplamenteEncadeado<T>();
	}

	public int retorneTamanho() {
		return tamanho;
	}

	public void substituaDaPosicao(T dado, int pos) throws ExcecaoEstruturaVazia,
			ExcecaoPosicaoIlegal {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		if (pos < 1 || pos > tamanho)
			throw new ExcecaoPosicaoIlegal(pos);

		encontrarElementoNaPosicao(pos).setDado(dado);
	}

	public void substituaDoFim(T dado) throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		ltimo.setDado(dado);
	}

	public void substituaDoInicio(T dado) throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		primeiro.setDado(dado);
	}

	public Lista<T> temIguais(T dado) {
		final Lista<T> listaDeIguais = new ListaDuplamenteEncadeada<T>();
		final Iterador<T> iter = retorneIterador();

		while (iter.temMais()) {
			final T dadoAtual = iter.retorneProximo();
			if (dadoAtual.equals(dado))
				listaDeIguais.insiraNoInicio(dadoAtual);
		}
		return listaDeIguais;
	}

	public T temIgual(T dado) {
		T retornado = null;
		final Iterador<T> iter = retorneIterador();
		while (retornado == null && iter.temMais()) {
			final T dadoAtual = iter.retorneProximo();
			if (dadoAtual.equals(dado))
				retornado = dadoAtual;
		}
		return retornado;
	}

}
