package ed20081.g1.tp1.validacao;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import ed20081.g1.tp1.enums.aleatorios.Estados;
import ed20081.g1.tp1.excecoes.sinistros.ExcecaoDataForaDoFormato;
import ed20081.g1.tp1.excecoes.sinistros.ExcecaoDataForaDosLimites;
import ed20081.g1.tp1.excecoes.sinistros.ExcecaoDataInvalida;
import ed20081.g1.tp1.excecoes.sinistros.ExcecaoEstadoInvalido;
import ed20081.g1.tp1.excecoes.sinistros.ExcecaoNumDeVitimasInvalido;

public class ValidadorDeSinistros extends Validador {

	private static ValidadorDeSinistros instanciaUnica;

	public static ValidadorDeSinistros obterInstancia() {
		if (instanciaUnica == null)
			instanciaUnica = new ValidadorDeSinistros();
		return instanciaUnica;
	}

	private final SimpleDateFormat sdf;

	private ValidadorDeSinistros() {
		sdf = new SimpleDateFormat("dd/MM/yyyy");
	}

	public Date validarData(String dataString) throws ExcecaoDataInvalida {

		Date data = null;
		Date minimo = null;
		final Date hoje = new Date();

		try {
			data = sdf.parse(dataString);
			minimo = sdf.parse("01/01/1885");
		} catch (final ParseException e) {
			throw new ExcecaoDataForaDoFormato();
		}

		if (data.after(hoje) || data.before(minimo))
			throw new ExcecaoDataForaDosLimites();

		return data;
	}

	public void validarEstado(String estado) throws ExcecaoEstadoInvalido {
		boolean estadoOk = false;
		final Estados[] e = Estados.values();
		for (int i = 0; i < e.length && !estadoOk; i++)
			if (estado.equalsIgnoreCase(e[i].toString()))
				estadoOk = true;
		if (!estadoOk)
			throw new ExcecaoEstadoInvalido();
	}

	public void validarVitimas(int numDeVitimas) throws ExcecaoNumDeVitimasInvalido {
		if (numDeVitimas < 0)
			throw new ExcecaoNumDeVitimasInvalido();
	}
}
